/*
 * $Id: RobotDisasterMain.java 322 2010-05-18 07:35:39Z mitsuki $
 *
 * Copyright 2009-2010 Fujitsu Limited.
 * FUJITSU CONFIDENTIAL.
 */
package sample.rsnp2x;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Logger;

import org.robotservices.v02.IAsyncCallBack;
import org.robotservices.v02.exception.RSiException;
import org.robotservices.v02.profile.common.Ret_value;
import org.robotservices.v02.profile.invoker.IBasic_profile;
import org.robotservices.v02.profile.invoker.IContents_profile;
import org.robotservices.v02.profile.invoker.InvokerProfileFactory;
import org.robotservices.v02.util.ConnectionInfo;

import com.fujitsu.rsi.helper.BasicProfileHelper;
import com.fujitsu.rsi.helper.ContentsProfileHelper;
import com.fujitsu.rsi.util.RESULT;

/**
 * 防災情報サービス　サンプル<br>
 * ロボットアプリメイン
 */
public class RobotDisasterMain {

	private static Logger log = Logger.getLogger(RobotDisasterMain.class
			.getName());

	private static String targetEPR = "http://rsnpcommon.aiit.ac.jp/RSi_Disaster-2.2/services";

	// ベーシック認証用ID/PW
	private static String basic_id = "rsiuser0";
	private static String basic_pw = "servicerobot";

	public static void main(String[] args) throws IOException {

		if (args.length < 2) {
			printUsage();
			return;
		}

		ServerSocket serversock = new ServerSocket(9000);
		Socket sock = null;

		// クエリー文をロード
		String requirementTerm = readFile("query.xml", "UTF-8");

		String user_id = args[0];
		String password = args[1];

		// 接続情報の設定
		ConnectionInfo ci = new ConnectionInfo();
		ci.set_endpointname(targetEPR);
		ci.set_username(basic_id);
		ci.set_password(basic_pw);
		// ci.set_proxy("localhost:8000");

		InvokerProfileFactory factory = null;
		IBasic_profile bp = null;
		IContents_profile cp = null;
		long conv_id = -1;
		try {
			factory = InvokerProfileFactory.newInstance(ci);
			// 通信経路を開設
			factory.connect();

			// Basic_profileによる認証
			bp = factory.getBasic_profile();
			Ret_value ret = bp.open(user_id, password);
			BasicProfileHelper helper = new BasicProfileHelper(ret);
			if (helper.getResult() != RESULT.SUCCESS.getResult()) {
				log.severe("Basic_profile#open()エラー[" + helper.getDetailCode()
						+ "]：" + helper.getDetail());
				return;
			}

			conv_id = helper.getConv_id();
			log.info("conv_id:" + conv_id);

			// 情報受信処理
			IAsyncCallBack callback = new IAsyncCallBack() {
				@Override
				public void doEvent(Ret_value ret, boolean isLast) {
					ContentsProfileHelper helper = new ContentsProfileHelper(
							ret);
					System.out.println(helper.getDetail());
				}

				@Override
				public void doException(Exception e) {
					log.severe(e.getMessage());
				}
			};

			// 情報取得プロファイル呼び出し
			cp = factory.getContents_profile();
			ret = cp.distribute_contents(conv_id, requirementTerm, -1, 0,
					callback);

			// 依頼結果
			ContentsProfileHelper cphelper = new ContentsProfileHelper(ret);
			log.info("Result:" + cphelper.getResult());
			log.info("Detail:" + cphelper.getDetail());

			// Socketに接続があるまで停止
			System.out.println("localhost の9000番ポートに接続すると終了します。");
			sock = serversock.accept();

			// 配信停止
			if (cp != null) {
				cp.stop_distribute_contents(conv_id, 0);
			}

			// 会話セッション終了
			if (bp != null) {
				bp.close(conv_id);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				// 通信経路を切断
				if (factory != null) {
					factory.disconnect();
				}
			} catch (RSiException e) {
				e.printStackTrace();
			}
			if (sock != null) {
				sock.close();
			}
			serversock.close();
		}

		log.info("exit");
	}

	/**
	 * 指定されたファイルを、指定されたエンコードで読み込みます。
	 *
	 * @param filename
	 *            ファイルパス
	 * @param encode
	 *            エンコード
	 * @return 読み込んだファイルの内容
	 */
	private static String readFile(String filename, String encode) {
		StringBuilder sb = new StringBuilder();
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(
					new FileInputStream(filename), encode));

			String line = null;
			while ((line = br.readLine()) != null) {
				sb.append(line);
			}

		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		String requirementTerm = sb.toString();
		return requirementTerm;
	}

	/**
	 * 使用方法を表示します。
	 */
	private static void printUsage() {
		System.out.println("使い方：");
		System.out
				.println("java sample.rsnp2x.RobotDisasterMain <robotid> <password> ");
		System.out.println("	<robotid>:ロボットアカウント");
		System.out.println("	<password>:パスワード");
	}

}
